%Fit BVM distribution
%input:  P        - 2D PDF which is the softmax(Hist*F)
%output: mean_idx - mean of the BVM in the form of log-chroma histogram index
function mean_idx = BivariateVoMises(P)
   
    dim = size(P, 1);
    unit = 2*pi / dim;
    
    %map [0, dim-1] to theta [0, 2*pi)
    theta = [0:unit:(2*pi - unit)];
    theta_t = transpose(theta);
    
    %compute the marginal distributions P(i, j)
    Pi = sum(P, 2);
    Pj = sum(P, 1);
    
    %Compute the marginal expectation of the sine and cosin of the angle
    yi = sum(Pi .* sin(theta_t), 'all');
    xi = sum(Pi .* cos(theta_t), 'all');
    
    yj = sum(Pj .* sin(theta), 'all');
    xj = sum(Pj .* cos(theta), 'all');
    
    mean_u = mod(atan2(yi, xi), 2*pi) * (dim/(2*pi)) + 1;
    mean_v = mod(atan2(yj, xj), 2*pi) * (dim/(2*pi)) + 1;
    
    mean_idx = [mean_u; mean_v];
end
