%Visualize the log-chroma histogram
%input:
%        hist      - the input log-chroma histogram
%        offset    - starting point of the histogram
%        bin_width - size of the bin
%        bin_num   - number of the bin
%output:
%        img       - visualized log-chroma histogram image
function img = HistVis(hist, offset, bin_width, bin_num)

    %convert histogram to weights
    hist = (hist ./ max(hist(:))).^0.5;

    %map bin index to actual values
    bins = offset + [0 : bin_width : (bin_num - 1) * bin_size];
    [u, v] = ndgrid(bins, bins);

    %convert (u,v) to (r,g,b) at each histogram location
    rgb = cat(3, -u, zeros(size(u)), -v);
    rgb = exp(bsxfun(@minus, rgb, max(rgb, [], 3)));
    rgb = bsxfun(@rdivide, rgb, max(rgb,[],3));

    %make log(u), log(v)=0 show the ref rgb color
    axis = find(bins == 0);
    hist(axis,:,:) = 1;
    hist(:,axis,:) = 1;

    %weight the (r, g, b) based on the hist
    img = bsxfun(@times, hist(:,:), rgb);
    imagesc(img);

end