%import testing images from Gehler-Shi dataset
%output:
%        test_images - array of struct. Each struct includes tinted test
%                      image in linear RGB space, ccm, ground-truth
%                      illuminant RGB values, and the original file name.
function test_images = ImportTestImg()

VISUALIZE = false;

testI_dir = 'data/shi_gehler/images/';
dirents = dir(fullfile(testI_dir, '*.png'));

cvfolds_filename = 'data/shi_gehler/preprocessed/GehlerShi/cvfolds.txt';
cvfoldidx = load(cvfolds_filename);
cvfoldidx = cvfoldidx(1:length(cvfoldidx));
is_test = sparse(1:length(cvfoldidx), cvfoldidx, true);

stats_size = [256, 384]; %projects.(project).SENSOR.STATS_SIZE;
    
test_images = {};
for idx  = 1:length(dirents)
    select = find(is_test(idx), 1, 'first');
    if isempty(select)
        continue
    end
    im_filename = fullfile(testI_dir, dirents(idx).name);
  
    [image, mask, camera_name, illuminant, CCM_input, exif] = ReadImage(im_filename);

if VISUALIZE
      figure(1);
      % Make a white-balanced linear image with the ground-truth illuminant.
      gains = max(illuminant) ./ illuminant;
      image_vis = bsxfun(@times, image, permute(gains(:), [2,3,1]));
      image_vis = ...
        reshape(reshape(image_vis, [], 3) * CCM_input', size(image_vis));
      image_vis = image_vis ./ max(image_vis(:));
      image_vis = ApplySrgbGamma(max(0, min(1, image_vis)));
      imagesc(image_vis);
      title([num2str(i_file), ' - input'])
      axis image off; drawnow;
end
  
    image_project = image;
    mask_project = mask;
    illuminant_project = illuminant;

    if ~isnan(stats_size)
      % Crop off the last row/column of the image so its size is divisible by 2,
      % if necessary.
      image_project = image_project(...
        1:floor(size(image_project,1)/2)*2, ...
        1:floor(size(image_project,2)/2)*2, :);

      mask_project = mask_project(...
        1:floor(size(mask_project,1)/2)*2, ...
        1:floor(size(mask_project,2)/2)*2, :);

      % Crop the image/mask to match the aspect ratio of the specified stats.
      im_size = [size(image_project,1), size(image_project,2)];
      scale = min(im_size ./ stats_size);
      crop_size = floor( (stats_size * scale)/2 ) * 2;
      image_project = image_project(...
        (im_size(1) - crop_size(1))/2 + [1:crop_size(1)], ...
        (im_size(2) - crop_size(2))/2 + [1:crop_size(2)], :);
      mask_project = mask_project(...
        (im_size(1) - crop_size(1))/2 + [1:crop_size(1)], ...
        (im_size(2) - crop_size(2))/2 + [1:crop_size(2)]);
      im_size = [size(image_project,1), size(image_project,2)];
    end

      % If the project does not specify the CCM of the sensor, we inheret
      % the CCM of this image from the Gehler-Shi dataset.
      CCM_project = CCM_input;
      image_canonized = image_project;
      illuminant_canonized = illuminant_project;
    
      % Check that the image and the stats have the same aspect ratio.
      assert(abs(log(im_size(2) / im_size(1)) ...
               - log(crop_size(2) / crop_size(1))) < 0.01)

      % Downsample the image according to the mask, and downsample the mask.
      % This downsample is weighted (or equivalently, done in homogenous
      % coordinates) so that masked pixels are ignored.
      downsample = @(x)imresize(x, stats_size, 'bilinear');

      image_down = downsample(image_canonized);

      max_val = max(image_down(:));
      image_down = min(1, image_down ./ max_val);

      if VISUALIZE
        figure(1);
        % Visualize the image by white balancing it and applying the CCM and
        % gamma curve (if necessary).
        vis = double(image_down) / double(max(image_down(:)));
        if ~projects.(project).SENSOR.LINEAR_STATS
          vis = UndoSrgbGamma(vis);
        end
        vis = bsxfun(@times, vis, permute(1./illuminant_canonized(:), [2,3,1]));
        vis = reshape(reshape(vis, [], 3) * CCM_project', size(vis));
        vis = max(0, vis / max(vis(:)));
        vis = ApplySrgbGamma(vis);
        imagesc(vis); axis image off;
        title([num2str(i_file), ' - ', project])
        drawnow;
      end

    test_images{idx} = struct(...
        'tinted_img', image_down, ...
        'rgb_gt', illuminant_canonized, ...
        'ccm', CCM_project, ...
        'original_filename', im_filename);

end

test_images= cat(1, test_images{:});
end

